/*
 *  $Source: w:/cvsroot/webarts/widgets/PlayList.java,v $
 *  $Name:  $
 *  $Revision: 1.2 $
 *  $Date: 2001/09/02 03:24:23 $
 *  $Locker:  $
 */
/*
 *  PlayList -- A helper widget class used with the jOggPlayer.
 *
 *  Written by Tom Gutwin - WebARTS Design.
 *  Copyright (C) 2001 WebARTS Design, North Vancouver Canada
 *  http://www..webarts.bc.ca/jOggPlayer
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package ca.bc.webarts.widgets;

import ca.bc.webarts.jOggPlayer;
import ca.bc.webarts.tools.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.lang.NullPointerException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import java.util.Vector;
import javax.swing.JCheckBox;

import javax.swing.JPanel;

/**
 * A simple class to encapsulate the storage of references to Ogg files in a Playlist
 * style.
 *
 * @author    Tom Gutwin P.Eng
 * @created   October 15, 2001
 */
public class PlayList
{

  /**
   * Description of the Field
   */
  private final static String SYSTEM_FILE_SEPERATOR = File.separator;
  /**
   * This is the meat of the class; it holds OggFileRef objects.*
   */
  private Vector songs = null;

  /**
   * A Class holder for its name (used in Logging).
   */
  private static String className_ = "PlayList";
  /**
   * The Log Filename.
   */
  private static String logFile_ = "." + File.separator + className_+ "Log.txt";
  /**
   * The Log that will get used.
   */
  protected static Log log_ = Log.createLog(Log.DEBUG, logFile_);




  /**
   * basic constructor to initialize things. *
   */
  public PlayList()
  {
    final String methodName = className_ + ": PlayList()";
    log_.startMethod(methodName);

    songs = new Vector();

    log_.endMethod();
  }


  /**
   * Constructor to get things init and load up an initial list of URLs *
   *
   * @param urls  Description of Parameter
   */
  public PlayList(URL[] urls)
  {
    final String methodName = className_ + ": PlayList(URL[])";
    log_.startMethod(methodName);

    songs = new Vector();
    //      System.out.println("*** Creating a NEW PlayList "+urls.length);
    if (urls != null)
    {
      for (int i = 0; i < urls.length; i++)
      {
        songs.add(new OggFileRef(urls[i]));
      }
    }
    log_.endMethod();
  }


  /**
   * Parses the passed filenames looking for OGG files. It will recurse directories
   * if a dir is in the passed array.
   *
   * @param fileNames  an array containg files and/or dir names to parse for OGG
   *      files.
   * @return           an array of OGG files that have been found within the passed
   *      in array of filenames.
   */
  public static String[] getPlaylistArray(String[] fileNames)
  {
    final String methodName = className_ + ": getPlaylistArray(String[])";
    log_.startMethod(methodName);

    String[] retVal = null;
    if (fileNames != null)
    {
      Vector v = getPlaylistVector(fileNames);
      int size = v.size();
      retVal = new String[size];
      log_.debug("Loading PlayList... "+size);
      if (size > 0)
      {
        for (int i = 0; i < size; i++)
        {
          retVal[i] = v.get(i).toString();
        }
      }
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Parses the passed filenames looking for OGG files. It will recurse directories
   * if a dir is in the passed array.
   *
   * @param fileNames  an array containg files and/or dir names to parse for OGG
   *      files.
   * @return           a vector of OGG files that have been found within the passed
   *      in array of filenames.
   */
  public static Vector getPlaylistVector(String[] fileNames)
  {
    final String methodName = className_ + ": getPlaylistVector(String[])";
    log_.startMethod(methodName);

    Vector retVal = new Vector();
    File f;
    if (fileNames != null)
    {
//      System.out.println("Loading Vector... "+fileNames.length);
      for (int i = 0; i < fileNames.length; i++)
      {
        f = new File(fileNames[i].trim());
        //System.out.print(" "+i+"> " +fileNames[i]);
        if (f.isDirectory())
        { // && checkBoxRecursePlaylist.isSelected() )

          // fill from dir
          File oggDir = new File(fileNames[i]);
//          playListDir_ = fileNames[i];
          String[] oggDirFiles = oggDir.list();
          for (int j = 0; j < oggDirFiles.length; j++)
          {
            oggDirFiles[j] = fileNames[i].trim() + SYSTEM_FILE_SEPERATOR +
              oggDirFiles[j];
          }
          // recurse on the dir
          //System.out.println(" is a Directory");
          retVal.addAll(getPlaylistVector(oggDirFiles));
        }
        else if (fileNames[i].trim().toUpperCase().endsWith(".OGG"))
        {
          //System.out.println(" is a File");
          retVal.addElement(fileNames[i]);
        }
      }
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Gets the SongNamesVector attribute of the PlayList object
   *
   * @return   The SongNamesVector value
   */
  public Vector getSongNamesVector()
  {
    final String methodName = className_ + ": getSongNamesVector()";
    log_.startMethod(methodName);

    int size = songs.size();
    Vector retVal = new Vector();
//      System.out.println("SongNamesVectorSize="+size);
    for (int i = 0; i < size; i++)
    {
      retVal.add(((OggFileRef) (songs.get(i))).name_);
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Gets the SongNames attribute of the PlayList object
   *
   * @return   The SongNames value
   */
  public String[] getSongNames()
  {
    final String methodName = className_ + ": getSongNames()";
    log_.startMethod(methodName);

    int size = songs.size();
    String[] retVal = new String[size];
    for (int i = 0; i < size; i++)
    {
      retVal[i] = ((OggFileRef) (songs.get(i))).name_;
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Gets the URL attribute of the PlayList object
   *
   * @param songName  Description of Parameter
   * @return          The URL value
   */
  public URL getURL(String songName)
  {
    final String methodName = className_ + ": getURL(String)";
    log_.startMethod(methodName);

    URL retVal = null;
    if (songName != null)
    {
      for (int i = 0; i < songs.size(); i++)
      {
        if (((OggFileRef) (songs.get(i))).name_.equals(songName))
        {
          retVal = ((OggFileRef) songs.get(i)).url_;
          break;
        }
      }
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Checks to see if the list of songs in this playlist is empty.
   *
   * @return   true if the list of songs in this playlist is empty, false if not.
   */
  public boolean isEmpty()
  {
    final String methodName = className_ + ": isEmpty()";
    log_.startMethod(methodName);

    log_.endMethod();
    return songs.isEmpty();
  }


  /**
   * Parses the passed filename looking for OGG files. It will recurse directories
   * if a dir is in the passed String.
   *
   * @param fileName  an String (a file or dir name) to parse for OGG files.
   * @return          a vector of OGG files that have been found within the passed
   *      in String.
   */
  public Vector getPlaylistVector(String fileName)
  {
    final String methodName = className_ + ": getPlaylistVector(String)";
    log_.startMethod(methodName);

    Vector retVal = new Vector();
    String[] temp = {fileName};
    if (fileName != null)
    {
      retVal = getPlaylistVector(temp);
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Gets the SongSelected attribute of the PlayList object
   *
   * @param songName  Description of Parameter
   * @return          The SongSelected value
   */
  public boolean isSongSelected(String songName)
  {
    final String methodName = className_ + ": isSongSelected(String)";
    log_.startMethod(methodName);

    log_.endMethod();
    return isSongSelected(findSong(songName));
  }


  /**
   * Gets the SongSelected attribute of the PlayList object
   *
   * @param songIndex  Description of Parameter
   * @return           The SongSelected value
   */
  public boolean isSongSelected(int songIndex)
  {
    final String methodName = className_ + ": isSongSelected(int)";
    log_.startMethod(methodName);

    log_.endMethod();
    return ((OggFileRef) songs.get(songIndex)).isSongSelected();
  }


  /**
   * Finds the song in the playlist with the requested name and returns the
   * index.
   *
   * @param songName  Description of Parameter
   * @return          Description of the Returned Value
   */
  public int findSong(String songName)
  {
    final String methodName = className_ + ": findSong(String[])";
    log_.startMethod(methodName);

    int retVal = -1;
    for (int i = 0; i < songs.size(); i++)
    {
      if (((OggFileRef) songs.get(i)).name_.equals(songName))
      {
        retVal = i;
        i = songs.size();
      }
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Validates/checks if the requested song is in the list.
   *
   * @param songName  Description of Parameter
   * @return          Description of the Returned Value
   */
  public boolean contains(String songName)
  {
    final String methodName = className_ + ": contains(String)";
    log_.startMethod(methodName);

    boolean retVal = false;
    int indx = findSong(songName);
    if (indx >= 0)
    {
      retVal = true;
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Adds the specified song to the playlist.
   *
   * @param url  the specification for the song to add.
   * @return     the OggFileRef name of the added song.
   */
  public String addSong(URL url)
  {
    final String methodName = className_ + ": addSong(URL)";
    log_.startMethod(methodName);

    OggFileRef oggFileRef = new OggFileRef(url);
    songs.add(oggFileRef);
    log_.endMethod();
    return oggFileRef.name_;
  }


  /**
   * Creates and returns a small JCheckBox with the selected song name and a small
   * checkbox indicating if the song is selected.
   *
   * @param songName  Description of Parameter
   * @return          a JCheckBox containing a checkbox and Name of the song.
   */
  public JCheckBox createSongCheckBox(String songName)
  {
    final String methodName = className_ + ": createSongCheckBox(String)";
    log_.startMethod(methodName);

    log_.endMethod();
    return createSongCheckBox(findSong(songName));
  }


  /**
   * Creates and returns a small JCheckBox with the selected song name and a small
   * checkbox indicating if the song is selected.
   *
   * @param index  the song num to look up
   * @return       a JCheckBox containing a checkbox and Name of the song.
   */
  public JCheckBox createSongCheckBox(int index)
  {
    final String methodName = className_ + ": createSongCheckBox(int)";
    log_.startMethod(methodName);

    OggFileRef oggRef = ((OggFileRef) songs.get(index));
    JCheckBox jc = new JCheckBox("uninit", false);
    if (oggRef != null)
    {
      jc.setText(oggRef.name_);
      jc.setSelected(oggRef.isSongSelected());
    }
    log_.endMethod();
    return jc;
  }


  /**
   * Creates and returns a small JPanel with the selected song name and a small
   * checkbox indicating if the song is selected. This Panel is suitable to use
   * a as a JComboBox Item.
   *
   * @param index  the song num to look up
   * @return       a JPanel containing a checkbox and Name of the song.
   */
  public JPanel createSongPanel(int index)
  {
    final String methodName = className_ + ": createSongPanel(int)";
    log_.startMethod(methodName);

    OggFileRef oggRef = ((OggFileRef) songs.get(index));
    JPanel retVal = new JPanel();
    retVal.add(new JCheckBox(oggRef.name_, oggRef.isSongSelected()));
    log_.endMethod();
    return retVal;
  }


  /**
   * Creates and returns a small JPanel with the selected song name and a small
   * checkbox indicating if the song is selected. This Panel is suitable to use
   * a as a JComboBox Item.
   *
   * @param song  the song name to look up
   * @return      a JPanel containing a checkbox and Name of the song.
   */
  public JPanel createSongPanel(String song)
  {
    final String methodName = className_ + ": createSongPanel(String)";
    log_.startMethod(methodName);

    OggFileRef oggRef = ((OggFileRef) songs.get(findSong(song)));
    JPanel retVal = new JPanel();
    retVal.add(new JCheckBox(oggRef.name_, oggRef.isSongSelected()));
    log_.endMethod();
   return retVal;
  }


  /**
   * The number of entries in the PlayList.
   *
   * @return   The number of entries in the PlayList.
   */
  public int size()
  {
    final String methodName = className_ + ": size()";
    log_.startMethod(methodName);

    int retVal = 0;
    if (songs != null)
    {
      retVal = songs.size();
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Sets the song 'selected' according to the passed boolean param.
   *
   * @param songIndex  Description of Parameter
   * @param select     Description of Parameter
   */
  public void selectSong(int songIndex, boolean select)
  {
    final String methodName = className_ + ": selectSong(int, boolean)";
    log_.startMethod(methodName);

    ((OggFileRef) songs.get(songIndex)).setSelected(select);
    log_.endMethod();
  }


  /**
   * Loads a playlist from the playlist file specified in the playListFilename_
   * field. It first trys to load each line as a URL; if that fails it loads them
   * as files. It simply adds the songs listed to the playList.
   *
   * @param filenameURL  the filename to use as the playlist - specified as a URL
   */
  public void loadPlaylist(URL filenameURL)
  {
    final String methodName = className_ + ": loadPlaylist(URL)";
    log_.startMethod(methodName);

    InputStream is = null;
    if (filenameURL != null)
    {
      /*
       *  get the file ready to read
       */
      try
      { // load it as a URL

        URLConnection urlc = filenameURL.openConnection(); // throws IOException
        is = urlc.getInputStream();

        /*
         *  Now load the songs listed
         */
        String line = null;
        // check each time if the is has not disappeared
        while (is != null && (line = readline(is)) != null)
        {
          try
          {
            addSong(new URL(line));
          }
          catch (MalformedURLException ee)
          {
            // NOTHING JUST GO TO NEXT
          }
        }
      }
      catch (MalformedURLException ee)
      {
        System.out.println("The Playlist Filename has some errors. " +
          "Please check the filename.");
      }
      catch (IOException ee)
      {
        System.out.println("The Playlist Filename cannot be read.");
      }
    }
    log_.endMethod();
  }



  /**
   * Saves the playlist to the file specified in the playListFilename_ field. <PRE>[playlist]
   * File1=H:\MP3s\U2\October\02 - I Fall Down.mp3
   * File2=H:\MP3s\Leonard Cohen\01 - Suzanne.mp3
   * NumberOfEntries=2
   * </PRE>
   *
   * @param filename  Description of Parameter
   */
  public void savePlaylistFile(String filename)
  {
    final String methodName = className_ + ": savePlaylistFile(String)";
    log_.startMethod(methodName);

    try
    {
      FileWriter out = new FileWriter(filename, false);
      String currUrlStr = "";
      URL[] playUrls = getSongUrls();
      out.write("[playlist]" +
        System.getProperty("line.separator", "\n"));
      String tempNumStr = "";
      int i;
      for (i = 0; i < playUrls.length; i++)
      {
        tempNumStr = "File" + (i + 1) + "=";
        currUrlStr = playUrls[i].toString();
        out.write(tempNumStr + currUrlStr +
          System.getProperty("line.separator", "\n"));
      }
      out.write("NumberOfEntries=" + i);
      out.close();
    }
    catch (IOException ex)
    {
      System.out.println("Could Not Save To Playlist File " + filename);
    }
    log_.endMethod();
  }


  /**
   * Description of the Method
   *
   * @param songName  Description of Parameter
   * @return          Description of the Returned Value
   */
  public boolean removeSong(String songName)
  {
    final String methodName = className_ + ": removeSong(String)";
    log_.startMethod(methodName);

    boolean retVal = false;
    int indx = findSong(songName);
    log_.debug("Attempting removal of song "+songName+" index="+indx);
    if (indx >= 0)
    {
      songs.removeElementAt(indx);
      retVal = true;
    }
    else
      log_.minor("Invalid songNmae for remaoval: "+songName);
    log_.endMethod();
    return retVal;
  }


  /**
   * Gets the SongUrls attribute of the PlayList object
   *
   * @return   The SongUrls value
   */
  protected URL[] getSongUrls()
  {
    final String methodName = className_ + ": getSongUrls()";
    log_.startMethod(methodName);

    int size = songs.size();
    URL[] retVal = new URL[size];
    for (int i = 0; i < size; i++)
    {
      retVal[i] = ((OggFileRef) songs.get(i)).url_;
    }
    log_.endMethod();
    return retVal;
  }


  /**
   * Reads a single line from the InputStream.
   *
   * @param is   Description of Parameter
   * @return     the line read as a String
   */
  private String readline(InputStream is)
  {
    final String methodName = className_ + ": readline(InputStream)";
    log_.startMethod(methodName);

    StringBuffer rtn = new StringBuffer();
    int temp;
    do
    {
      try
      {
        temp = is.read();
      }
      catch (Exception e)
      {
        return (null);
      }
      if (temp == -1)
      {
        return (null);
      }
      if (temp != 0 && temp != '\n')
      {
        rtn.append((char) temp);
      }
    } while (temp != '\n');

    log_.endMethod();
    return (rtn.toString());
  }
}




/**
 * A simple class to encapsulate the information required to use/reference an Ogg
 * file.
 *
 * @author    Tom Gutwin P.Eng
 * @created   October 15, 2001
 */
class OggFileRef
{


  /**
   * Description of the Field
   */
  public String name_ = "";
  /**
   * Description of the Field
   */
  public URL url_ = null;
  /**
   * Description of the Field
   */
  private boolean selected_ = true;


  /**
   * Constructor for the OggFileRef object
   *
   * @param newUrl  Description of Parameter
   */
  OggFileRef(URL newUrl)
  {
    url_ = newUrl;
    name_ = url_.toString().substring(url_.toString().lastIndexOf('/') + 1);
  }


  /**
   * Sets the Selected attribute of the OggFileRef object
   *
   * @param s  The new Selected value
   */
  public void setSelected(boolean s)
  {
    selected_ = s;
  }


  /**
   * Gets the SongSelected attribute of the OggFileRef object
   *
   * @return   The SongSelected value
   */
  public boolean isSongSelected()
  {
    return selected_;
  }
}
/*
 *  Here is the revision log
 *  ------------------------
 *  $Log: PlayList.java,v $
 *  Revision 1.2  2001/09/02 03:24:23  tgutwin
 *  Initial Rev of my custom DropDown Boxes.
 *  Revision 1.1  2001/08/12 06:00:01  tgutwin
 *  Intial Revision. Includes OGG files Only.
 */


