/* ************************************************************************** */
/*    Splash                                                                  */
/*  - Tom B. Gutwin -                                                         */
/*  En Extended Frame to act as a splash pane that comes up before the app    */
/*  ************************************************************************* */
/*  This program requires JVM 1.2 or JVM 1.1  SWING NOT REQUIRED              */
/*  If building with the OS/2 JDK - at least 1.1.4 with the 1.1.4 fixes       */
/*  ************************************************************************* */
/*
 *  Copyright (C) 2001 Tom Gutwin
 *  tgutwin@webarts.bc.ca
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the jEdit program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/*
 *  $Copyright $
 *  $Header: w:/cvsroot/webarts/widgets/Splash.java,v 1.3 2001/10/28 06:48:28 tgutwin Exp $
 *  $Log: Splash.java,v $
 *  Revision 1.3  2001/10/28 06:48:28  tgutwin
 *  Added some Constructors that take an image (so this class doesn't have to go looking for the file.
 *  Revision 1.2  2001/09/02 03:24:30  tgutwin
 *  Initial Rev of my custom DropDown Boxes.
 *  Revision 1.1.1.1  2001/03/03 23:36:06  tgutwin
 *  Imported Sources
 */
package ca.bc.webarts.widgets;

import ca.bc.webarts.widgets.ImageCanvas;
import ca.bc.webarts.tools.Log;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Toolkit;
import java.awt.Window;
import java.io.File;

/**
 * Splash is a AWT based Application Opening splash screen. It provides
 * an expandable API for its use as a generic Splash, allowing the user to
 * specify the graphic to use as well as the message to put up.
 *
 * @author    Tom Gutwin P.Eng
 * @created   October 29, 2001
 */
public class Splash extends Frame implements Runnable
{
  /**
   * The filename for the Image to display.
   */
  static String imgName = "." + File.separator +
    "images" + File.separator + "dont_pan.gif";
  /**
   * The time the splash stays on the screen (in msec.).
   */
  static int mSec = 3000;
  /**
   * The default amount of time the splash stays on the screen (in msec.).
   */
  final static int DEFAULT_SLEEP_TIME = 2000;
  /**
   * A panel that holds a little message below the graphic
   */
  static Panel messageArea;
  /**
   * The default java awt toolkit.
   */
  private Toolkit toolkit = Toolkit.getDefaultToolkit();
  /**
   * The main Splash window.
   */
  Window window;
  /**
   * The Image being displayed.
   */
  Image image = null;
  /**
   * A boolen flag to signal the splash sleep thread.
   */
  boolean splashFinished = false;
  /**
   * A Class holder for its name (used in Logging).
   */
  private static String className_ = "Splash";
  /**
   * The Log Filename.
   */
  private static String logFile_ = "." + File.separator + className_+ "Log.txt";
  /**
   * The Log that will get used.
   */
  protected static Log log_ = Log.createLog(Log.FULL, logFile_);


  /**
   * Constructor for the Splash object using the default sleep time.
   */
  public Splash()
  {
    final String methodName = className_ + ": Constructor()";
    log_.startMethod(methodName);

    mSec = DEFAULT_SLEEP_TIME;
    (new Thread(this)).start();

    log_.endMethod();
  }


  /**
   * Constructor for the Splash object using the passed image name using
   * the default splash sleep time.
   *
   * @param imageName  The name of the file to load for the Splash Image.
   */
  public Splash(String imageName)
  {
    final String methodName = className_ + ": Constructor(Image)";
    log_.startMethod(methodName);

    mSec = DEFAULT_SLEEP_TIME;
    imgName = imageName;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object using the passed image using the default
   * splash sleep time.
   *
   * @param img  The Splashed Image.
   */
  public Splash(Image img)
  {
    final String methodName = className_ + ": Constructor(Image)";
    log_.startMethod(methodName);

    image = img;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object
   *
   * @param imageName  The name of the file to load for the Splash Image.
   * @param time          The time in mSec that the Splash Window stays on the
   *                      screen.
   */
  public Splash(String imageName, int time)
  {
    final String methodName = className_ + ": Constructor(String, int)";
    log_.startMethod(methodName);

    System.out.println("Splashing " + imgName);
    imgName = imageName;
    mSec = time;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object
   *
   * @param img  The Splashed Image.
   * @param time          The time in mSec that the Splash Window stays on the
   *                      screen.
   */
  public Splash(Image img, int time)
  {
    final String methodName = className_ + ": Constructor(Image, int)";
    log_.startMethod(methodName);

   image = img;
    mSec = time;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object
   *
   * @param imageName  The name of the file to load for the Splash Image.
   * @param time          The time in mSec that the Splash Window stays on the
   *                      screen.
   * @param messagePanel  Description of Parameter
   */
  public Splash(String imageName, int time, Panel messagePanel)
  {
    final String methodName = className_ + ": Constructor(String, int, Panel)";
    log_.startMethod(methodName);

    imgName = imageName;
    mSec = time;
    messageArea = messagePanel;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object
   *
   * @param img           The Splashed Image.
   * @param time          The time in mSec that the Splash Window stays on the
   *                      screen.
   * @param messagePanel  Description of Parameter
   */
  public Splash(Image img, int time, Panel messagePanel)
  {
    final String methodName = className_ + ": Constructor(Image, int, Panel)";
    log_.startMethod(methodName);

    image = img;
    mSec = time;
    messageArea = messagePanel;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * Constructor for the Splash object
   *
   * @param imageName  The name of the file to load for the Splash Image.
   * @param messagePanel  A panel that gets placed below the image(can hold a
   *                      message.
   */
  public Splash(String imageName, Panel messagePanel)
  {
    final String methodName = className_ + ": Constructor(String, Panel)";
    log_.startMethod(methodName);

    imgName = imageName;
    mSec = DEFAULT_SLEEP_TIME;
    messageArea = messagePanel;
    (new Thread(this)).start();
    log_.endMethod();
  }


  /**
   * The main program for the Splash class
   *
   * @param args  The command line arguments
   */
  public static void main(String[] args)
  {
    System.out.println("Splashing " + imgName);
    if (args.length > 0)
    {
      imgName = args[0];
    }
    if (args.length > 1)
    {
      mSec = (Integer.valueOf(args[1])).intValue();
    }
    Frame frame = new Splash(imgName, mSec);
    System.exit(0);
  }


  /**
   * Description of the Method
   */
  public void finishSplash()
  {
    final String methodName = className_ + ": finishSplash()";
    log_.startMethod(methodName);

    splashFinished = true;
    log_.endMethod();
  }


  /**
   * Description of the Method
   */
  public void indicateUpdate()
  {
    final String methodName = className_ + ": indicateUpdate()";
    log_.startMethod(methodName);

    int counter = 0;
    // an unimplemented update indicator of some type.
    log_.endMethod();
  }


  /**
   * Main processing method for the Splash object.
   */
  public void run()
  {
    final String methodName = className_ + ": run()";
    log_.startMethod(methodName);

    Thread.currentThread().setPriority(Thread.MAX_PRIORITY);

    ImageCanvas canvas;

    window = new Window(this);
    if (image == null)
    {
      System.out.println("Splashing " + imgName);
      log_.debug("Loading the Image File:"+imgName);
      image = toolkit.getImage(imgName);
    }
    log_.debug("Placing the image "+(image==null)+"on the imageCnavas");
    canvas = new ImageCanvas(image);
    window.setLayout(new BorderLayout());

    window.add("Center", canvas);

    if (messageArea != null)
    {
      window.add("South", messageArea);
    }

    Dimension scrnSize = toolkit.getScreenSize();
    int imgWidth = image.getWidth(this);
    int imgHeight = image.getHeight(this);
    int
      panelHeight = messageArea.getPreferredSize().height * 3 + 3;

    window.setLocation(scrnSize.width / 2 - (imgWidth / 2),
      scrnSize.height / 2 - ((imgHeight + panelHeight) / 2));
    window.setSize(imgWidth, imgHeight + panelHeight);
    window.pack();
    window.show();
    window.toFront();
    window.setVisible(true);
    this.repaint();
    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
    while (!splashFinished)
    {
      try
      {
        Thread.currentThread().sleep(mSec);
        finishSplash();
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

    window.dispose();
    log_.endMethod();
  }
}




