/*
 *  $Source: w:/cvsroot/webarts/widgets/Util.java,v $
 *  $Name:  $
 *
 *  $Revision: 1.6 $
 *  $Date: 2001/12/16 07:15:37 $
 *  $Locker:  $
 *
 *
 *  Written by Tom Gutwin - WebARTS Design.
 *  Copyright (C) 2001 WebARTS Design, North Vancouver Canada
 *  http://www.webarts.bc.ca
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package ca.bc.webarts.widgets;

import java.awt.Component;
import java.awt.Image;
import javax.swing.ImageIcon;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.JFileChooser;

import ca.bc.webarts.tools.Log;

/**
 * A set of useful utility methods/functions for reuse.
 *
 * @author    Tom Gutwin P.Eng
 * @created   October 4, 2001
 */
public class Util
{
  /**
   * A holder for this clients System File Separator.
   */
  public final static String SYSTEM_FILE_SEPERATOR = File.separator;

  /**
   * A constant to specify which dialog to open with the chooseFileDialog.
   */
  public final static short OPEN_DIALOG = 0;

  /**
   * A constant to specify which dialog to open with the chooseFileDialog.
   */
  public final static short SAVE_DIALOG = 1;

  /**
   * A holder This classes name (used when logging).
   */
  private  static String CLASSNAME = "ca.bc.webarts.widgets.Util";


  /**
   * The VM classpath (used in some methods)..
   */
  private static String CLASSPATH = System.getProperty("java.class.path");


  /**
   * Gets a Log instance in case any calling classes are using it.
   **/
  private static Log log_ = Log.getInstance();


  /**
   * Class flag signifying if the initUtil method has been called
   **/
  private static boolean initUtil = false;

  synchronized private static void initUtil()
  {
    if (!initUtil)
    {
      log_ = Log.getInstance();
      CLASSPATH = System.getProperty("java.class.path");
      CLASSNAME = "ca.bc.webarts.widgets.Util";
      initUtil = true;
    }
  }

  /**
   * Ensures that a folder exists.
   * <TABLE WIDTH="100%" BORDER="0" CELLSPACING="1" CELLPADDING="2">
   *  <TR><TD COLSPAN="2">
   *   <H2>Ensures that a folder exists</H2>
   *    </TD></TR>
   *
   *    <TR><TD COLSPAN="2">&nbsp;<BR>
   *    <B>Description:</B><BR>
   *    use it like this:
   *  <br>    ensureFolderExists(new File(fileName).getParentFile());
   *    </TD></TR>
   *    </TABLE>
   *
   * @param folder  The File object to check.
   */
public static void ensureFolderExists(File folder)
{
  if ( folder != null  &&  ! folder.exists() )
  {
      ensureFolderExists(folder.getParentFile());
      folder.mkdir();
  }
}


  /**
   * A simple byte to Hex String converter.
   *
   * @param b  The byte to convert
   * @return   The resultant Hex String
   */
  public static String toHEXString(byte b)
  {
    return ("" + "0123456789ABCDEF".charAt(0xf & b >> 4) +
            "0123456789ABCDEF".charAt(b & 0xF));
  }


  /**
   * Creates a timestamp for the current time in the form of
   * 'hour + "-" + min + "-" + sec + "-" + millis'.
   *
   * @return   The CurrentTimeStamp value.
   */
  public static String createCurrentTimeStamp()
  {
    String value = "";
    Calendar calendar_ = Calendar.getInstance();
    int currMillis = calendar_.get(calendar_.MILLISECOND);
    String millis = String.valueOf(currMillis);
    if (currMillis < 10)
    {
      millis = "00" + currMillis;
    }
    else if (currMillis < 100)
    {
      millis = "0" + currMillis;
    }
    int currSec = calendar_.get(calendar_.SECOND);
    String sec = String.valueOf(currSec);
    if (currSec < 10)
    {
      sec = "0" + currSec;
    }
    int currMin = calendar_.get(calendar_.MINUTE);
    String min = String.valueOf(currMin);
    if (currMin < 10)
    {
      min = "0" + currMin;
    }
    int currHr = calendar_.get(calendar_.HOUR_OF_DAY);
    String hour = String.valueOf(currHr);
    if (currHr < 10)
    {
      hour = "0" + currHr;
    }

    return (String) hour + "-" + min + "-" + sec + "-" + millis;
  }


  /**
   * A simple String token replacement routine. Replaces all occurences of the
   * token parameter with the replacement value in the passed in sentence
   * parameter.
   *
   * @param sentence     The String to perform the token replacement on
   * @param token        the token String to seartch for and replace
   * @param replacement  the tokens replacement value
   * @return             The new token replaced string
   */
  public static String tokenReplace(String sentence,
    String token,
    String replacement)
  {
    initUtil();
    final String methodName = CLASSNAME + ": tokenReplace("+
    sentence + " " +token+ " " + replacement;
    if(log_!=null) log_.startMethod(methodName);
    int a = 0;
    String retVal = "";
    while ((a = sentence.indexOf(token)) > -1)
    {
      retVal += sentence.substring(0, a) + replacement;
      sentence = sentence.substring(a + token.length());
      // System.out.println(wort);
    } // if a > -1
    retVal += sentence;
    if(log_!=null) log_.endMethod();
    return retVal;
  }


  /**
   * Recursively deletes all the files in the spec'd directory and
   * subdirectories, and then removes the dir.
   *
   * @param fileName  The filename of the directory to delete.
   */
  public static void removeDir(String fileName)
  {
    initUtil();
    File dirToDel = initDirFile(fileName);
    if (dirToDel != null)
    {
      String[] files = dirToDel.list();
      File tempFile;

      //loop through all the files in the dir DELETING them
      for (int i = 0; i < files.length; i++)
      {
        tempFile = new File(fileName + File.separator + files[i]);
        if (tempFile.isDirectory())
        {
          // recurse on the subdirectory
          removeDir(fileName + File.separator + files[i]);
        }
        else
        {
          // Delete all the files
          tempFile.delete();
        }
      }
      dirToDel.delete();
    }
  }


  /**
   * Initisalizes File only if it is directory.It is represented by the
   * passed in String.
   *
   * @param s  the directory name to init as a File
   * @return   An instatiated File object if the passed string is a dir, null if
   *           not
   */
  public static File initDirFile(String s)
  {
    initUtil();
    File retVal = null;
    File dirFile = new File(s);
    if (dirFile.isDirectory() && dirFile.canRead())
    {
      retVal = dirFile;
    }
    return dirFile;
  }


/**
   * Searches the classpath for the specified filename and then returns the full
   * path that is used for it.
   *
   * @param filename to look for in the classpath
   * @param fallBack if the file is not found in classpath
   * @return the path description of the passed filename as found in the
   *         classpath.
   **/
  public static String getFilePathFromClasspath(String filename, String fallBack)
  {
    initUtil();
    final String methodName = CLASSNAME +
      ": getFilePathFromClasspath(String, String)";
    if(log_!=null) log_.startMethod(methodName);
    String retVal = getFilePathFromClasspath(filename);
    System.out.println("File In Classpath="+retVal);
    if (retVal == null || retVal.equals(""))
      retVal = fallBack;
    if(log_!=null)
      log_.endMethod();
    System.out.println("File In Classpath="+retVal);
    return retVal;
  }


/**
   * Searches the classpath for the specified filename and then returns the full
   * path that is used for it.
   *
   * @return the path description of the passed filename as found in the
   *         classpath.
   **/
  public static String getFilePathFromClasspath(String filename)
  {
    initUtil();
    final String methodName = CLASSNAME + ": getFilePathFromClasspath";
    if(log_!=null) log_.startMethod(methodName);
    String retVal = "";
    String pathSep = System.getProperty("path.separator");
    if (CLASSPATH == null)
      return "";
    int fileIndex = CLASSPATH.indexOf(filename);
    if(log_!=null)
      log_.debug("Searching the Classpath for " + filename +
          " " +fileIndex);
    if (fileIndex>=0)
    {
      int startSpot = 0;
      int nextSpot = CLASSPATH.indexOf(pathSep);
      // the fileName is in the classpath
      if(log_!=null) log_.debug("Start Looking at ("+startSpot+","+nextSpot+")");
      while ((nextSpot < fileIndex) && nextSpot != -1)
      {
        startSpot = nextSpot;
        nextSpot = CLASSPATH.indexOf(pathSep,startSpot+1);
        if(log_!=null) log_.debug(", ("+startSpot+","+nextSpot+")");
      }
      if (startSpot <= 0) startSpot = -1;
      if (nextSpot <= 0) nextSpot = CLASSPATH.length() ;
      retVal = CLASSPATH.substring(startSpot+1, nextSpot);
    }
    else
      if(log_!=null)
        log_.minor(filename+" cannot be found in classpath.");
    if(log_!=null)
      log_.endMethod();
    return retVal;
  }

  /**
   * Loads the requested Image filename from the specified jarfile.
   *
   * @param imageFilename the filename in the jarfile to use as the basis for
   *        the Image to retrieve.
   * @param jarFilename the jar file to look through for the image.
   * @return Image the Image loaded from the filename spec'd as an imput param,
   *         It returns null if it cannot retrieve the Image.
   **/
  public static Image loadImage(String imageFilename, String jarFilename)
  {
    initUtil();
    final String methodName = CLASSNAME + ": loadImage()";
    if(log_!=null) log_.startMethod(methodName);
    if(log_!=null) log_.debug("Loading Imagename /"+imageFilename);
    Image retVal = (new ImageIcon(Util.class.getResource("/"+imageFilename))).
      getImage();
    /*
    String iconJarFileName = jarFilename; //getFilePathFromClasspath(jarFilename);
    if(log_!=null) log_.debug("Loading Imagename "+imageFilename+" from Jarfile "+jarFilename);
    InputStream in = null;
    JarEntry tempJarEntry = null;
    byte [] imageBytes;
    long imageFileSize = 0l;
    int bytesRead = 0;
    try
    {
      JarFile jar = new JarFile(iconJarFileName, false);
      tempJarEntry = jar.getJarEntry(imageFilename);
      in = jar.getInputStream(tempJarEntry);
      imageFileSize = tempJarEntry.getSize();
      imageBytes = new byte[(int)imageFileSize];
      for (int j=0;j<imageFileSize;j++)
        imageBytes[j] = new Integer(in.read()).byteValue();
      if(log_!=null) log_.debug("Loaded "+imageFileSize+" Bytes.");
      retVal = (Toolkit.getDefaultToolkit()).createImage(imageBytes);
    }
    catch (NullPointerException fEx)
    {
      System.out.println("No graphicJarEntry NullPointerException " +
                          imageFilename);
    }
    catch (FileNotFoundException fEx)
    {
      System.out.println("graphic FileNotFoundException " +imageFilename);
    }
    catch (IOException ioEx)
    {
      System.out.println("graphic FileNotFoundException " +imageFilename);
    }
    catch (SecurityException secEx)
    {
      System.out.println("graphic SecurityException " +imageFilename);
    }
    */
    if(log_!=null) log_.endMethod();
      return retVal;
  }
  /**
   * Loads the requested Image filename from the classpath.
   *
   * @param imageFilename the filename in the jarfile to use as the basis for
   *        the Image to retrieve.
   * @return Image the Image loaded from the filename spec'd as an imput param,
   *         It returns null if it cannot retrieve the Image.
   **/
  public static Image loadImage(String imageFilename)
  {
    initUtil();
    final String methodName = CLASSNAME + ": loadImage()";
    if(log_!=null) log_.startMethod(methodName);
    if(log_!=null) log_.debug("Loading Imagename /"+imageFilename);
    Image retVal = (new ImageIcon(Util.class.getResource("/"+imageFilename))).
      getImage();

    if(log_!=null) log_.endMethod();
      return retVal;
  }


  /**
   * A method to simply abstract the Try/Catch required to put the current
   * thread to sleep for the specified time in ms.
   *
   * @param  waitTime  the sleep time in milli seconds (ms).
   *
   * @return boolean value specifying if the sleep completed (true) or was
   *                 interupted (false).
   **/
  public static boolean sleep(long waitTime)
  {
    final String methodName = CLASSNAME + ": sleep(Long)";
    if(log_!=null) log_.startMethod(methodName);
    boolean retVal = true;
    /* BLOCK for the spec'd time */
    try
    {
      Thread.sleep(waitTime);
    }
    catch (InterruptedException iex)
    {
      retVal = false;
    }
    if(log_!=null) log_.endMethod();
      return retVal;
  }


  /**
   * Encapsulates the showing of the FileChooser dialog and returns a String
   * representing the absolute path. <P> NO file filters will be used and NO
   * directory selection is allowed.
   *
   * @param parent the owner of this dialog (generally just send this)
   * @param startPath the dir to start the file chooser dialog from
   * @return the absolute path of the chosen file.
   **/
  public static String chooseAFilename(Component parent, String startPath)
  {
    initUtil();
    final String methodName = CLASSNAME + ": chooseAFilename(String)";
    if(log_!=null) log_.startMethod(methodName);
    if(log_!=null) log_.endMethod();
      return chooseAFilename(parent, startPath, null, false);
  }


  /**
   * Encapsulates the showing of the FileChooser dialog and returns a String
   * representing the absolute path.<P>This method requires all the config parms
   * specified by the ExampleFileFilter.
   *
   * @param parent the owner of this dialog (generally just send this)
   * @param startPath the dir to start the file chooser dialog from
   * @param filters an array holding the Filters to use in the dialog
   * @param filesAndDirs specs if the user will be allowed to choose both files
   *        or dirs.
   * @return the absolute path of the chosen file.
   **/
  public static String chooseAFilename(Component parent,
                                 String startPath,
                                 ExampleFileFilter[] filters,
                                 boolean filesAndDirs
                                 )
  {
    return chooseAFilename(parent, startPath, filters,
                           filesAndDirs, Util.OPEN_DIALOG);
  }


  /**
   * Encapsulates the showing of the FileChooser dialog and returns a String
   * representing the absolute path.<P>This method requires all the config parms
   * specified by the ExampleFileFilter.
   *
   * @param parent the owner of this dialog (generally just send this)
   * @param startPath the dir to start the file chooser dialog from
   * @param filters an array holding the Filters to use in the dialog
   * @param filesAndDirs specs if the user will be allowed to choose both files
   *        or dirs.
   * @return the absolute path of the chosen file.
   **/
  public static String chooseAFilename(Component parent,
                                 String startPath,
                                 ExampleFileFilter[] filters,
                                 boolean filesAndDirs,
                                 short openOrClose)
  {
    initUtil();
    final String methodName = CLASSNAME +
      ": chooseAFilename(String,ExampleFileFilter[],boolean)";
    if(log_!=null) log_.startMethod(methodName);
    String retVal = "";
    JFileChooser chooser = new JFileChooser();
    if (filters != null && filters.length >0)
    {
      for (int i=0; i<filters.length; i++)
      {
        chooser.addChoosableFileFilter(filters[i]);
      }
      chooser.setFileFilter(filters[0]);
    }
    if (filesAndDirs)
      chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    if (openOrClose == SAVE_DIALOG)
    {
      if(chooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION)
      {
        retVal = chooser.getSelectedFile().getAbsolutePath();
        System.out.println("You chose this file: " +retVal);
      }
    }
    else
    {
      if(chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION)
      {
        retVal = chooser.getSelectedFile().getAbsolutePath();
        System.out.println("You chose this file: " +retVal);
      }
    }
    if(log_!=null) log_.endMethod();
      return retVal;
  }


  /**
   * Creates a URL for the specified filename string.
   *
   * @param fileString is the filename to turn into a file://URL
   * @return the converted file://URL
   */
  public static URL getFileBaseURL(String fileString)
  {
    initUtil();
    final String methodName = CLASSNAME + ": getFileBaseURL(String)";
    if(log_!=null) log_.startMethod(methodName);
    /* because this is an application the codebase does not work */
    /* we have to create the codeBase URL from the userDir */
    URL convertedURL = null;
    int i = 0;
    String tempPath = null;
    try
    {
      // 1st see if the string is already a URL
      convertedURL = new URL(fileString);
    }
    catch(MalformedURLException ex)
    {
      try
      {
        try
        {
          /* 1st test if this is a DOS/OS/2 style path with a drive letter */
          if (fileString.charAt(1) == ':')
          {
            tempPath = fileString.substring(0,1);
            tempPath += ":";
            /* Replace the file separators with the forward slash */
            for ( i = 2; i <= fileString.length() -1; i++)
            {
              if (fileString.charAt(i) != SYSTEM_FILE_SEPERATOR.charAt(0) )
              {
                tempPath += fileString.substring(i,i+1);
              }
              else
              {
                tempPath += "/";
              }
            }
           // System.out.println("Appears to be a DOS or OS/2 filesystem");
            convertedURL = new URL("FILE:/" + tempPath);
          }
          else
          {
            /* 2nd test if this is a UN*x style path without a drive letter */
            if (fileString.charAt(0) == '/')
            {
              //System.out.println("Appears to be a Unix filesystem");
              convertedURL = new URL("FILE:" + fileString);
            }
            else
            {
              convertedURL = new URL("FILE:/./");
            }
          }
          //  System.out.println("APP convertedURL=" + convertedURL.toString());
        }
        catch(java.net.MalformedURLException t)
        {
          System.out.println("caught java.net.MalformedURLException: " + t +
            "Cannot retrieve User Directory");
          t.printStackTrace();
        }
      }
      catch(StringIndexOutOfBoundsException t)
      {
          System.out.println("caught StringIndexOutOfBoundsException: " + t +
            "Cannot retrieve User Directory");
          t.printStackTrace();
      }
    }
    if(log_!=null) log_.endMethod();
      return convertedURL;
  }
}


